// Common/StdOutStream.cpp

#include "StdAfx.h"

#include "StdOutStream.h"
#include "Common/IntToString.h"
#include "Common/StringConvert.h"

static const char kNewLineChar =  '\n';

CStdOutStream  g_StdOut;

CStdOutStream ::CStdOutStream ()
{
	m_lpOutBuf = (char*)::GlobalAlloc(GMEM_FIXED, 4096);
	m_lpCommandLine = NULL;
	ReSet();
}

CStdOutStream ::~CStdOutStream ()
{
	::GlobalFree(m_lpOutBuf);
	if (m_lpCommandLine)
		free(m_lpCommandLine);
}

CStdOutStream & CStdOutStream::operator<<(CStdOutStream & (*aFunction)(CStdOutStream  &))
{
  (*aFunction)(*this);    
  return *this;
}

CStdOutStream & endl(CStdOutStream & outStream)
{
  return outStream << kNewLineChar;
}

CStdOutStream & CStdOutStream::operator<<(const char *string)
{
	char* lpszText = (char*)_alloca(strlen(string) * 2);
	char* p = lpszText;
	while (*string)
	{
		// s"\r""\r\n"ɏC
		if (*string == '\n')
		{
			*p = '\r';
			++p;
		}
		*p = *string;
		++p;
		++string;
	}
	*p = '\0';
	OutProgress(lpszText);
	return *this;
}

CStdOutStream & CStdOutStream::operator<<(const wchar_t *string)
{
  *this << (const char *)UnicodeStringToMultiByte(string, CP_OEMCP);
  return *this;
}

CStdOutStream & CStdOutStream::operator<<(char c)
{
	char lpszText[3];
	if (c == '\n')
	{
		lpszText[0] = '\r';
		lpszText[1] = '\n';
		lpszText[2] = '\0';
	}
	else
	{
		lpszText[0] = c;
		lpszText[1] = '\0';
	}
	OutProgress(lpszText);
	return *this;
}

CStdOutStream & CStdOutStream::operator<<(int number)
{
	char textString[16];
	ConvertINT64ToString(number, textString);
	return operator<<(textString);
}

CStdOutStream & CStdOutStream::operator<<(UINT64 number)
{
  char textString[32];
  ConvertUINT64ToString(number, textString);
  return operator<<(textString);
}

// i󋵂o
void CStdOutStream::OutProgress(const char *lpszText)
{
	int nStrLen = strlen(lpszText);
	if (m_dwMaxBuf <= m_dwBufSize + nStrLen)
	{
		m_dwMaxBuf += 4096;
		m_lpOutBuf = (char*)::GlobalReAlloc(m_lpOutBuf, m_dwMaxBuf, GMEM_MOVEABLE);
	}
	memcpy(m_lpOutBuf + m_dwBufSize, lpszText, nStrLen + 1);
	m_dwBufSize += nStrLen;
}


// oϐ
void CStdOutStream::ReSet()
{
	m_pDlgProgress = NULL;
	m_hThread = NULL;
	m_lpOutBuf = (char*)::GlobalReAlloc(m_lpOutBuf, 4096, GMEM_MOVEABLE);
	m_dwBufSize = 0;
	m_dwMaxBuf = 4096;
}

// o͂Rs[
// lpCopyBufNULL̎͏o̓obt@̃|C^̂Ԃ
LPCSTR CStdOutStream::CopyBuf(LPSTR lpCopyBuf, DWORD dwBufSize)
{
	if (lpCopyBuf == NULL)
		return m_lpOutBuf;
	strncpy(lpCopyBuf, m_lpOutBuf, dwBufSize);
	lpCopyBuf[dwBufSize - 1] = '\0';
	return lpCopyBuf;
}

// ꂽR}hL
void CStdOutStream::SetCommandLine(int argc, LPCSTR const* argv)
{
	UString strCmdLine = L"7-zip32.dll";
	for (int i = 0; i < argc; ++i)
	{
		strCmdLine += L" \"" + GetUnicodeString(argv[i]) + L"\"";
	}
	if (m_lpCommandLine)
		free(m_lpCommandLine);
	m_lpCommandLine = _wcsdup(strCmdLine);
}

LPWSTR CStdOutStream::MyGetCommandLineW()
{
	return m_lpCommandLine;
}