#include "stdafx.h"
#include "MainAr.h"
#include "SplitCmdLine.h"
#include "resource.h"

// #include <locale.h>

#include "Windows/COM.h"
#include "Windows/Error.h"

#include "Common/StdOutStream.h"
#include "Common/NewHandler.h"
#include "Common/StringConvert.h"

#include "../../UI/Console/ConsoleClose.h"	// pXύX
#include "../../UI/Console/ArError.h"	// pXύX

#include "../../UI/Console/OpenCallback.h"	// ǉ
#include "../../UI/Common/OpenArchive.h"	// ǉ

using namespace NWindows;

// extern int Main2(int numArguments, const char *arguments[]);
extern int Main2();

static const char *kExceptionErrorMessage = "\n\nError:\n";
static const char *kUserBreak  = "\nBreak signaled\n";

static const char *kMemoryExceptionMessage = "\n\nMemory Error! Can't allocate!\n";
static const char *kUnknownExceptionMessage = "\n\nUnknown Error\n";
static const char *kInternalExceptionMessage = "\n\nInternal Error #";

static const int kErrorCode[23][2] = { NExitCode::kSuccess, 0,
									   NExitCode::kWarning, ERROR_WARNING,
									   NExitCode::kFatalError, ERROR_FATAL,
									   NExitCode::kCRCError, ERROR_FILE_CRC,
									   NExitCode::kLockedArchive, ERROR_READ_ONLY,
									   NExitCode::kWriteError, ERROR_CANNOT_WRITE,
									   NExitCode::kOpenError, ERROR_FILE_OPEN,
									   NExitCode::kUserError, ERROR_COMMAND_NAME,
									   NExitCode::kMemoryError, ERROR_ENOUGH_MEMORY,
									   
									   NExitCode::kNotSupported, ERROR_NOT_ARC_FILE,
									   NExitCode::kFileError, ERROR_NOT_ARC_FILE,
									   
									   NExitCode::kVerError, ERROR_VERSION,
									   NExitCode::kMethodError, ERROR_METHOD,
									   
									   NExitCode::kUserQuit, ERROR_METHOD, 
									   
									   NExitCode::kFileIsNotArchive, ERROR_NOT_ARC_FILE,
									   
									   NExitCode::kCommonError, ERROR_FATAL,
									   
									   NExitCode::kInputArchiveException, ERROR_NOT_FIND_ARC_FILE,
									   
									   NExitCode::kErrorsDuringDecompression, ERROR_DURING_DECOMPRESSION,
									   
									   NExitCode::kDirFileWith64BitSize, ERROR_DIR_FILE_WITH_64BIT_SIZE,
									   NExitCode::kFileTimeWinToDosConvertError, ERROR_CONVERT_TIME,
									   
									   NExitCode::kFileChangedDuringOperation, ERROR_FILE_CHANGED_DURING_OPERATION,
									   
									   NExitCode::kUserBreak, ERROR_USER_CANCEL,
									   -1, -1 };


static int ExitCodeToErrorCode(int nExitCode)
{
	int nErrorCode = ERROR_FATAL;
	for (int i = 0; kErrorCode[i][0] != -1; ++i)
	{
		if (kErrorCode[i][0] == nExitCode)
		{
			nErrorCode = kErrorCode[i][1];
			break;
		}
	}
	return nErrorCode;
}

UINT _stdcall Main(LPVOID lpParameter)
{
	// R}hC𕪊ăC֓n
	CSplitCmdLine* pslc = (CSplitCmdLine*)lpParameter;
	int result = SendCommand7zip(pslc->m_argc, (const char**)pslc->m_argv);
	g_StdOut.CloseProgressDialog();

	return ExitCodeToErrorCode(result);
}

static inline bool IsItWindowsNT()
{
  OSVERSIONINFO versionInfo;
  versionInfo.dwOSVersionInfoSize = sizeof(versionInfo);
  if (!::GetVersionEx(&versionInfo)) 
    return false;
  return (versionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
}

int SendCommand7zip(int argc, const char* argv[])
{
  #ifdef UNICODE
  if (!IsItWindowsNT())
  {
    g_StdOut << "This program requires Windows NT/2000/XP";
    return NExitCode::kFatalError;
  }
  #endif
  // setlocale(LC_COLLATE, ".OCP");
  int result=1;
  NCOM::CComInitializer comInitializer;
	try
	{
		NConsoleClose::CCtrlHandlerSetter aCtrlHandlerSetter;
		try
		{
			// result = Main2(numArguments, arguments);
			result = Main2();
		}
		catch(const NConsoleClose::CCtrlBreakException &)
		{
			throw  NExitCode::CSystemError(E_ABORT);
//			g_StdOut << endl << kUserBreak;
//			return(ERROR_USER_CANCEL);
		}
	}
	catch(const CNewException)
	{
		g_StdOut << kMemoryExceptionMessage;
		return(NExitCode::kMemoryError);
	}
	catch(const CSystemException &e)
	{
		g_StdOut << "System Error: " << (UINT64)(e.ErrorCode);
		return (NExitCode::kFatalError);
	}
	catch(NExitCode::EEnum &aExitCode)
	{
		g_StdOut << kInternalExceptionMessage << aExitCode << endl;
		return(aExitCode);
	}
	catch(const NExitCode::CSystemError &systemError)
	{
		UINT32 ErrorValue = systemError.ErrorValue;
		UString message;
		NError::MyFormatMessage(systemError.ErrorValue, message);
		switch(ErrorValue)
		{
		case E_ABORT:
			g_StdOut << endl << endl << message << endl;
			return NExitCode::kUserBreak;
		default:
			g_StdOut << endl << endl << "System error:" << endl << 
			message << endl;
			return(NExitCode::kFatalError);
		}
	}
	catch(const NExitCode::CMultipleErrors &multipleErrors)
	{
		g_StdOut << endl << multipleErrors.NumErrors << " errors" << endl;
		return(NExitCode::kCRCError);
	}
	catch(const UString &s)
	{
		g_StdOut << kExceptionErrorMessage << s << endl;
		return(NExitCode::kFatalError);
	}
	catch(const char *s)
	{
		g_StdOut << kExceptionErrorMessage << s << endl;
		return(NExitCode::kFatalError);
	}
	catch(int t)
	{
		g_StdOut << kInternalExceptionMessage << t << endl;
		return(NExitCode::kFatalError);
	}
	catch(...)
	{
		g_StdOut << kUnknownExceptionMessage;
		return(NExitCode::kFatalError);
	}
	if (argc == 1)
		return NExitCode::kUserError;
	return result;
}

// t@CΉĂ邩`FbNk̎ނ擾
//  S_OK / Ή S_FALSE / ȊÕG[ E_FAIL ԂB
HRESULT CheckArchive(LPCSTR lpFileName, LPSTR lpMode)
{
    CMyComPtr<IInArchive> archiveHandler;
    CArchiverInfo archiverInfo;
    COpenCallbackImp *openCallbackSpec = new COpenCallbackImp;
    CMyComPtr<IArchiveOpenCallback> openCallback = openCallbackSpec;
	int subExtIndex;

	HRESULT res = OpenArchive(GetUnicodeString(lpFileName), &archiveHandler, archiverInfo, subExtIndex, openCallback);
	if (lpMode)
		strcpy(lpMode, GetOemString(archiverInfo.Name));
	return res;
}
